#!/bin/bash
set -e
set -o pipefail

export TABLES_PATH="$SCHEMA_DIR/tables"
export PRE_UPGRADE_PATH="$SCHEMA_DIR/pre_upgrade"
export RELATIONS_PATH="$SCHEMA_DIR/relations"
export POST_UPGRADE_PATH="$SCHEMA_DIR/post_upgrade"
export DATA_PATH="$SCHEMA_DIR/data"
export TEMPLATES_PATH="$SCHEMA_DIR/templates"

__exec_pre_upgrade() {
  log_proc_marker "Executing pre-upgrade scripts"

  while read pre_upgrade_script; do
    local file_path_sql="${PRE_UPGRADE_PATH}/${pre_upgrade_script}.sql"
    local file_path_js="${PRE_UPGRADE_PATH}/${pre_upgrade_script}.js"
    if [ -f "$file_path_sql" ]; then
      local pre_upgrade_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
        -v ON_ERROR_STOP=1 \
        -a -f $file_path_sql > /dev/null"
      eval "$pre_upgrade_cmd"
    elif [ -f "$file_path_js" ]; then
      local file_path="$JF_PIPELINES_HOME/installer/scripts/schema/pre_upgrade/$pre_upgrade_script".js
      local pre_upgrade_cmd="DBCONNECTIONSTRING=$PIPELINES_SHARED_DB_CONNECTIONSTRING \
        node $JF_PIPELINES_HOME/installer/scripts/migrations.js $file_path"
      eval "$pre_upgrade_cmd"
    else
      log_proc_error "No file exists with supported extension: $pre_upgrade_script"
    fi
  done < $PRE_UPGRADE_PATH/order.txt
}

__init_tables() {
  log_proc_marker "Initializing tables"

  for file_path in $TABLES_PATH/*.sql; do
    local init_table_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
      -v ON_ERROR_STOP=1 \
      -a -f $file_path > /dev/null"
    eval "$init_table_cmd"
  done
}

__init_relations() {
  log_proc_marker "Initializing relations"

  while read table_name; do
    file_path="${RELATIONS_PATH}/${table_name}.sql"
    if [ -f "$file_path" ]; then
      local init_relation_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
        -v ON_ERROR_STOP=1 \
        -a -f $file_path > /dev/null"
      eval "$init_relation_cmd"
    fi
  done < $RELATIONS_PATH/order.txt
}

__exec_post_upgrade() {
  log_proc_marker "Executing post-upgrade scripts"

  while read post_upgrade_script; do
    local file_path_sql="${POST_UPGRADE_PATH}/${post_upgrade_script}.sql"
    local file_path_js="${POST_UPGRADE_PATH}/${post_upgrade_script}.js"
    if [ -f "$file_path_sql" ]; then
      local post_upgrade_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
        -v ON_ERROR_STOP=1 \
        -a -f $file_path_sql > /dev/null"
      eval "$post_upgrade_cmd"
    elif [ -f "$file_path_js" ]; then
      local file_path="$JF_PIPELINES_HOME/installer/scripts/schema/post_upgrade/$post_upgrade_script".js
      local post_upgrade_cmd="DBCONNECTIONSTRING=$PIPELINES_SHARED_DB_CONNECTIONSTRING \
        PIPELINEVERSION=$PIPELINES_CORE_CONTROLPLANEVERSION \
        node $JF_PIPELINES_HOME/installer/scripts/migrations.js $file_path"
      eval "$post_upgrade_cmd"
    fi
  done < $POST_UPGRADE_PATH/order.txt
}

__init_data() {
  log_proc_marker "Initializing data"

  while read table_name; do
    file_path="${DATA_PATH}/${table_name}.sql"
    if [ -f "$file_path" ]; then
      local init_data_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
        -v ON_ERROR_STOP=1 \
        -a -f $file_path > /dev/null"
      eval "$init_data_cmd"
    fi
  done < $DATA_PATH/order.txt

  local version="$PIPELINES_CORE_BUILDPLANEVERSION"
  if [ ! -z "$BUILDPLANE_UPGRADE_VERSION" ]; then
    version="$BUILDPLANE_UPGRADE_VERSION"
  fi

  log_proc_msg "Updating intendedBuildPlane version for all nodes"
  local update_nodes_sql="update nodes set \"intendedBuildPlaneVersion\"='\'$version\'';"
  local update_nodes_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
    -v ON_ERROR_STOP=1 \
    -a -c  '$update_nodes_sql' > /dev/null"
  eval "$update_nodes_cmd"

  log_proc_msg "Updating build plane images"
  local BUILDPLANE_IMAGES_FILE="$INSTALLER_BIN/buildplaneImages.json"
  if [ ! -f "$BUILDPLANE_IMAGES_FILE" ]; then
    log_proc_warn "Build plane images file doesnt exist, skipping update"
    return
  fi

  local build_plane_images=$(cat $BUILDPLANE_IMAGES_FILE)
  if [ -z "$build_plane_images" ]; then
    log_proc_warn "Empty build plane images, skipping update"
    return
  fi

  #TODO:
  # get this from db based on the masterintegration set in json
  # this is currently being set to AWS master integration ID
  
  local build_plane_images_count=$(echo $build_plane_images \
    | jq '. | length')
  for i in $(seq 1 $build_plane_images_count); do
    local build_plane_image=$(echo $build_plane_images \
      | jq '.['"$i-1"']')
    local name=$(echo $build_plane_image \
      | jq -r '.name')
    local image_id=$(echo $build_plane_image \
      | jq -r '.imageId')
    local region=$(echo $build_plane_image \
      | jq -r '.region')
    local architecture="$(echo $build_plane_image \
      | jq -r '.architecture')"
    local operating_system="$(echo $build_plane_image \
      | jq -r '.os')"
    local created_at="$(echo $build_plane_image \
      | jq -r '.createdAt')"
    local provider="$(echo $build_plane_image \
      | jq -r '.provider')"
    local metadata_property_bag="$(echo $build_plane_image \
      | jq -rc '.metadataPropertyBag')"
    local get_master_integration_sql="select id from \"masterIntegrations\" where name = '\'$provider\'';"
    local get_master_integration_cmd="psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
      -v ON_ERROR_STOP=1 \
      -t -c '$get_master_integration_sql'"
    local masterIntegration_id=$(eval "$get_master_integration_cmd")
    if [ ! -z "$masterIntegration_id" ]; then
      local add_build_plane_image_sql="INSERT into \"buildPlaneImages\"
        (name, \"masterIntegrationId\", \"imageId\", \"isAvailable\", architecture, \"operatingSystem\", region, \"metadataPropertyBag\", \"createdAt\", \"updatedAt\")
        VALUES ('$name', '$masterIntegration_id', '$image_id', 'true', '$architecture', '$operating_system', '$region', '$metadata_property_bag', '$created_at', '$created_at')
        ON CONFLICT (\"name\", \"region\", \"operatingSystem\",\"architecture\",\"masterIntegrationId\") DO UPDATE SET \"imageId\" = EXCLUDED.\"imageId\", \"metadataPropertyBag\" = EXCLUDED.\"metadataPropertyBag\";"

      psql $PIPELINES_SHARED_DB_CONNECTIONSTRING \
        -v ON_ERROR_STOP=1 \
        -a -c  "$add_build_plane_image_sql" > /dev/null
    else
      log_proc_error "No masterIntegration found for provider name: $provider"
      exit 1
    fi
  done
}

exec_db_task() {
  local db_task="$@"
  case $db_task in
    initTables)
      log_proc_msg "Initializing object tables"
      __init_tables
      ;;
    preUpgrade)
      log_proc_msg "Executing pre upgrade scripts"
      __exec_pre_upgrade
      ;;
    initRelations)
      log_proc_msg "Initializing object relations"
      __init_relations
      ;;
    postUpgrade)
      log_proc_msg "Executing post upgrade scripts"
      __exec_post_upgrade
      ;;
    initData)
      log_proc_msg "Initializing seed data"
      __init_data
      ;;
    *)
      log_proc_error "Invalid db task: $db_task"
      exit 1
      ;;
  esac
}

## Fixed one for GCP dynamic nodes
